--
-- Package declaration: 
--
-- Written by: 
-- Date:
-- Short description...
--
LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_UNSIGNED.all;
--
PACKAGE stopper_pkg IS
--
COMPONENT counter_bcd
	PORT( 
		Clk			: IN STD_LOGIC;
		Rst			: IN STD_LOGIC;
		En			: IN STD_LOGIC;
		--
		Digit		: OUT STD_LOGIC_VECTOR(4-1 downto 0);
		Carry	 	: OUT STD_LOGIC
	);
END COMPONENT;

COMPONENT stopper
	PORT( 
		Clk		: IN  STD_LOGIC;
		Rst		: IN  STD_LOGIC;
		En		: IN  STD_LOGIC;
		--
		Digit_0	: OUT STD_LOGIC_VECTOR(3 downto 0);
		Digit_1	: OUT STD_LOGIC_VECTOR(3 downto 0);
		Digit_2	: OUT STD_LOGIC_VECTOR(3 downto 0);
		Digit_3	: OUT STD_LOGIC_VECTOR(3 downto 0);
		Carry 	: OUT STD_LOGIC
	);
END COMPONENT;
--
END PACKAGE;

-- 
-- Implementing the components in the package
--

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.std_logic_UNSIGNED.all;
USE work.tuw.ALL;

ENTITY counter_bcd IS
	PORT( 
		Clk			: IN STD_LOGIC;
		Rst			: IN STD_LOGIC;
		En			: IN STD_LOGIC;
		--
		Digit		: OUT STD_LOGIC_VECTOR(4-1 downto 0);
		Carry	 	: OUT STD_LOGIC
	);
END counter_bcd;

ARCHITECTURE counter_bcd_arch OF counter_bcd IS

-- Local signal declarations
SIGNAL cnt_val			: STD_LOGIC_VECTOR(4-1 downto 0);
SIGNAL carry_val		: STD_LOGIC;

BEGIN

-- Output strappings
Digit		<= cnt_val;
Carry		<= carry_val;

-- Main process
Ucp: PROCESS(clk, Rst, En, cnt_val)
BEGIN
IF (clk'event AND clk='1') THEN -- everything must be sychronous!
--
	IF (Rst = '1') THEN -- Master reset
		RESET(cnt_val);
		carry_val		<= '0';
	ELSIF (En = '1') THEN -- Count when Enable is HI
		IF (cnt_val = "1000") THEN -- 8. issue carry for the next digit
			carry_val	<= '1';
		ELSE
			carry_val	<= '0';
		END IF;
		--
		IF (carry_val = '1') THEN -- 9. reset digit counter
			RESET(cnt_val);
		ELSE
			INCR(cnt_val);
		END IF;
	END IF;
--
END IF;
END PROCESS;

END counter_bcd_arch;


LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE work.stopper_pkg.ALL;

ENTITY stopper IS
	PORT( 
		Clk		: IN  STD_LOGIC;
		Rst		: IN  STD_LOGIC;
		En		: IN  STD_LOGIC;
		--
		Digit_0	: OUT STD_LOGIC_VECTOR(3 downto 0);
		Digit_1	: OUT STD_LOGIC_VECTOR(3 downto 0);
		Digit_2	: OUT STD_LOGIC_VECTOR(3 downto 0);
		Digit_3	: OUT STD_LOGIC_VECTOR(3 downto 0);
		Carry 	: OUT STD_LOGIC
	);
END stopper;

ARCHITECTURE stopper_arch OF stopper IS
--
SIGNAL carry_0			: STD_LOGIC;
SIGNAL carry_1			: STD_LOGIC;
SIGNAL carry_2			: STD_LOGIC;
SIGNAL carry_3			: STD_LOGIC;
--
SIGNAL En_0			: STD_LOGIC;
SIGNAL En_1			: STD_LOGIC;
SIGNAL En_2			: STD_LOGIC;
SIGNAL En_3			: STD_LOGIC;
--
BEGIN

--
-- Simple chain of BCD counters
--

En_0		<= En;

Udig1 : counter_bcd
	PORT MAP(
		clk		=> clk,
		Rst		=> Rst,
		En		=> En_0,
		--
		Digit	=> Digit_0,
		Carry	=> carry_0
	);

En_1		<= En_0 AND carry_0;

Udig2 : counter_bcd	
	PORT MAP(
		clk		=> clk,
		Rst		=> Rst,
		En		=> En_1,
		--
		Digit	=> Digit_1,
		Carry	=> carry_1
	);

En_2		<= En_0 AND carry_0 AND carry_1;

Udig3 : counter_bcd	
	PORT MAP(
		clk		=> clk,
		Rst		=> Rst,
		En		=> En_2,
		--
		Digit	=> Digit_2,
		Carry	=> carry_2
	);

En_3		<= En_0 AND carry_0 AND carry_1 AND carry_2;

Udig4 : counter_bcd	
	PORT MAP(
		clk		=> clk,
		Rst		=> Rst,
		En		=> En_3,
		--
		Digit	=> Digit_3,
		Carry	=> carry_3
	);

Carry	<= En_0 AND carry_0 AND carry_1 AND carry_2 AND carry_3;

END stopper_arch;

